// a few text replacements for converting gvim colorschemes
// to xterm-256 compatible ones
// Wolfgang Frisch, xororand@frexx.de 
//
// - helps to convert gvim colorschemes to 256 color ones,
//   reading such themes from stdin.
// - reads the X rgb.txt file, replaces named/rgb-hex colors
// - renames guifb/guibg to ctermfb/ctermbg

#include "conv.h"
#include <iostream>
#include <iomanip>
#include <fstream>
#include <sstream>
#include <map>
#include <cctype>
#include <stdlib.h>
using namespace std;

string stringreplace(string str, string from, string to)
{
	string ret;
	string::size_type s = str.find(from);
	if(s!=string::npos)
	{
		ret  = str.substr(0,s);
		ret += to;
		ret += str.substr(s+from.length());
		return ret;
	}
	return str;
}

// reads the /usr/X11R6/lib/X11/rgb.txt color map
void readcolormap(map<string,string>& colormap)
{
	ifstream ifstr("/usr/X11R6/lib/X11/rgb.txt");
	string line;
	
	if(ifstr.fail())
	{
		cerr << "loading /usr/X11R6/lib/X11/rgb.txt failed" << endl;
		exit(1);
	}
	
	while(getline(ifstr,line))
	{
		stringstream ss;
		unsigned int r=0,g=0,b=0;
		string colorname(""), dump("");

		if(line[0]=='!')
			continue;

		ss.str(line);
		ss >> r >> g >> b >> colorname >> dump;
		// only use names without spaces
		if(dump.length()==0)
		{
			stringstream fss;
			fss << setbase(16) << setfill('0') << setw(2) << r << setw(2) << g << setw(2) << b;
			// lower name
			for(size_t i=0; i<colorname.length(); i++)
				colorname[i]=tolower(colorname[i]);
			colormap[colorname]=fss.str();
		}
	}
}

int main(int argc, char** argv)
{
	string line;
	stringstream ss;
	unsigned char rgb[3];
	string tagswithcolors[2]={"guifg=","guibg="};
	string tagreplacement[2]={"ctermfg=","ctermbg="};
	bool is_named_color=false;
	string colorstring, rgbstring;
	
	// color map for named colors
	map<string,string> namedcolors;
	readcolormap(namedcolors);

	// replace stuff line by line
	while( getline(cin,line) )
	{
		line = stringreplace(line,"gui=","cterm=");

		// replace guifg and guibg
		for(int i=0; i<2; i++)
		{
			string::size_type s=0;
			while( (s=line.find(tagswithcolors[i],s)) != string::npos)
			{
				// color found, convert it
				unsigned short int colorcode;
				
				line.replace(s,tagswithcolors[i].length(),tagreplacement[i]);
				s+=tagreplacement[i].length();
				
				// cut out color string
				if(line[s]=='#') { line.erase(s,1); is_named_color=false; }
				else is_named_color=true;
				string colorstring = line.substr(s,line.find_first_of("\t ",s)-s);
				line.erase(s,line.find_first_of("\t ",s)-s);

				// lowercase it
				for(size_t x=0; x<colorstring.size(); x++)
					colorstring[x] = tolower(colorstring[x]);

				if(is_named_color)
				{
					// named color
					map<string,string>::iterator it;
					it = namedcolors.find(colorstring);
					if(it==namedcolors.end())
					{
						cerr << "unknown color: \"" << colorstring << "\"" << endl;
						continue;
					}
					// convert to rgb hex color
					rgbstring = (*it).second;
				}
				else
					rgbstring = colorstring;
				
				// rgb hex color
				readcolor(rgbstring.c_str(),rgb);
				colorcode = rgb2xterm(rgb);
				ss.str(""); 
				ss << colorcode << "\t";
				line.insert(s,ss.str());
			}
	
		}
		cout << line << endl;
	}

	return 0;
}

/*  old color map
 * 	namedcolors["black"]        = "000000";
	namedcolors["brown"]		= "af5f00";
	namedcolors["darkgreen"]	= "00cd00";
	namedcolors["lightgrey"]	= "e5e5e5";
	namedcolors["darkgrey"]		= "6c6c6c";
	namedcolors["red"]          = "ff0000";
	namedcolors["green"]		= "00ff00";
	namedcolors["yellow"]		= "ffff00";
	namedcolors["blue"]			= "0000ff";
	namedcolors["magenta"]		= "ff00ff";
	namedcolors["cyan"]			= "00ffff";
	namedcolors["white"]		= "ffffff";
	namedcolors["grey"]			= "a8a8a8";
	namedcolors["lightyellow"]  = "ffffaf";
	namedcolors["lightblue"]	= "5fd7ff";
	namedcolors["lightcyan"]	= "afffff";
	namedcolors["lightred"]     = "FFBBBB";
	namedcolors["lightgreen"]   = "88FF88";
	namedcolors["lightmagenta"] = "FFBBFF";
	namedcolors["darkcyan"]     = "008888";
	namedcolors["darkblue"]     = "0000BB";
	namedcolors["darkred"]      = "BB0000";
	namedcolors["darkmagenta"]  = "BB00BB";
	namedcolors["darkgrey"]     = "BBBBBB";
	namedcolors["darkyellow"]   = "BBBB00";*/
	
